'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = combineReducers;

var _createStore = require('redux/lib/createStore');

var _warning = require('redux/lib/utils/warning');

var _warning2 = _interopRequireDefault(_warning);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NODE_ENV = typeof process !== 'undefined' ? process.env.NODE_ENV : 'development';

function getUndefinedStateErrorMessage(key, action) {
  var actionType = action && action.type;
  var actionName = actionType && '"' + actionType.toString() + '"' || 'an action';

  return 'Given action ' + actionName + ', reducer "' + key + '" returned undefined. ' + 'To ignore an action, you must explicitly return the previous state. ' + 'If you want this reducer to hold no value, you can return null instead of undefined.';
}

function assertReducerSanity(reducers) {
  Object.keys(reducers).forEach(function (key) {
    var reducer = reducers[key];
    var initialState = reducer(undefined, { type: _createStore.ActionTypes.INIT });

    if (typeof initialState === 'undefined') {
      throw new Error('Reducer "' + key + '" returned undefined during initialization. ' + 'If the state passed to the reducer is undefined, you must ' + 'explicitly return the initial state. The initial state may ' + 'not be undefined. If you don\'t want to set a value for this reducer, ' + 'you can use null instead of undefined.');
    }

    var type = '@@redux/PROBE_UNKNOWN_ACTION_' + Math.random().toString(36).substring(7).split('').join('.');
    if (typeof reducer(undefined, { type: type }) === 'undefined') {
      throw new Error('Reducer "' + key + '" returned undefined when probed with a random type. ' + ('Don\'t try to handle ' + _createStore.ActionTypes.INIT + ' or other actions in "redux/*" ') + 'namespace. They are considered private. Instead, you must return the ' + 'current state for any unknown actions, unless it is undefined, ' + 'in which case you must return the initial state, regardless of the ' + 'action type. The initial state may not be undefined, but can be null.');
    }
  });
}

function removeUnexpectedState(lastState, reducer) {
  Object.keys(lastState).forEach(function (key) {
    if (!reducer[key]) {
      delete lastState[key];
    }
  });
  return lastState;
}

function combineReducers(reducers) {

  var finalReducer = {};

  Object.keys(reducers).forEach(function (key) {
    var reducer = reducers[key];
    if (NODE_ENV !== 'production') {
      if (typeof reducer === 'undefined') {
        (0, _warning2.default)('No reducer provider for key "' + key + '"');
      }
    }

    if (typeof reducer === 'function') {
      finalReducer[key] = reducer;
    }
  });

  var finalReducerKeys = Object.keys(finalReducer);

  var sanityError = void 0;
  try {
    assertReducerSanity(finalReducer);
  } catch (e) {
    sanityError = e;
  }

  return function rootReducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments[1];


    if (sanityError) {
      throw sanityError;
    }

    removeUnexpectedState(state, finalReducer);

    var hasChanged = false;
    var nextState = {};

    finalReducerKeys.forEach(function (key) {
      var reducer = finalReducer[key];
      var previousStateForKey = state[key];
      var nextStateForKey = reducer(previousStateForKey, action);
      if (typeof nextStateForKey === 'undefined') {
        var errorMessage = getUndefinedStateErrorMessage(key, action);
        throw new Error(errorMessage);
      }
      nextState[key] = nextStateForKey;
      hasChanged = hasChanged || nextStateForKey !== previousStateForKey;
    });

    return hasChanged ? nextState : state;
  };
}
module.exports = exports['default'];